// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// The anaMACDBBLines (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
	/// Optimized execution by predefining instances of external indicators (Zondor August 10 2010)
	/// </summary>
	[Description("The anaMACDBBLines (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.")]
	public class anaMACDBBLines : Indicator
	{
		#region Variables
		private int		bandperiod 		= 10;
		private int		fast			= 12;
		private int		slow			= 26;
		private int		smooth			= 5;
		private int		dotsize			= 2;
		private double	stdDevNumber	= 1.0;
		private Color	dotsUp			= Color.Lime;	
		private Color 	dotsDown		= Color.Red;
		private Color	dotsRim			= Color.Black;
		private Color	bbAverage		= Color.LightSkyBlue;
		private Color	bbUpper			= Color.LightSkyBlue;
		private Color	bbLower			= Color.LightSkyBlue;
		private Color	zeroPositive	= Color.Lime;
		private Color	zeroNegative	= Color.Red;
		private Color 	zeroCross		= Color.Yellow;
		private Color	connector		= Color.White;
		private bool 	init 			= false;
			
		private MACD BMACD;
		private EMA  EMAMACD;
		private StdDev SDBB;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.Black, 2), PlotStyle.Dot, "BBMACD"));
			Add(new Plot(new Pen(Color.Black, 3), PlotStyle.Dot, "BBMACDFrame"));
			Add(new Plot(new Pen(Color.Black, 1), PlotStyle.Line, "Average"));
			Add(new Plot(new Pen(Color.Black, 1), PlotStyle.Line, "Upper"));
			Add(new Plot(new Pen(Color.Black, 1), PlotStyle.Line, "Lower"));
			Add(new Plot(new Pen(Color.Black, 2), PlotStyle.Line, "ZeroLine"));
			Add(new Plot(new Pen(Color.Black, 8), PlotStyle.Dot, "MACDCross"));
			Add(new Plot(new Pen(Color.Black, 2), PlotStyle.Line, "BBMACDLine"));
			
			Plots[0].Pen.DashStyle = DashStyle.Dot;
			Plots[1].Pen.DashStyle = DashStyle.Dot;
			Plots[2].Pen.DashStyle = DashStyle.Dot;
			PlotsConfigurable = false;
			PaintPriceMarkers = false;
		}

		protected override void OnStartUp()
		{
			Plots[0].Pen.Width = DotSize;
			Plots[1].Pen.Width = DotSize +1;
			Plots[1].Pen.Color = DotsRim;
		}

		/// <summary>
		/// Calculates the indicator value(s) at the current index.
		/// </summary>
		protected override void OnBarUpdate()
		{
			if(!init)
			{
				BMACD=MACD(Input,fast,slow,smooth);
				EMAMACD=EMA(BMACD,bandperiod);
				SDBB=StdDev(BMACD,bandperiod);
			}
			init=true;
			double macdValue = BMACD[0];
			BBMACD.Set(macdValue);
			BBMACDLine.Set(macdValue);
			BBMACDFrame.Set(macdValue);
			
			//double avg = EMA(BBMACD,bandperiod)[0];
			double avg = EMAMACD[0];
			Average.Set(avg);
			ZeroLine.Set(0);

			//double stdDevValue = StdDev(BBMACD,bandperiod)[0];
			double stdDevValue = SDBB[0];
			Upper.Set(avg + StdDevNumber * stdDevValue);
			Lower.Set(avg - StdDevNumber * stdDevValue);
			
			if (Rising(BBMACD))
				PlotColors[0][0] = DotsUp;
			else
				PlotColors[0][0] = DotsDown;
			
			PlotColors[2][0] = BBAverage;
			PlotColors[3][0] = BBUpper;
			PlotColors[4][0] = BBLower;
			PlotColors[6][0] = ZeroCross;
			PlotColors[7][0] = Connector;

			if (BBMACD[0] > 0)
			{
				PlotColors[5][0] = ZeroPositive;
				if (CurrentBar != 0 && BBMACD[1] <= 0)
					MACDCross.Set(0);
				else
					MACDCross.Reset();
			}
			else
			{
				PlotColors[5][0] = ZeroNegative;
				if (CurrentBar != 0 && BBMACD[1] > 0)
					MACDCross.Set(0);
				else
					MACDCross.Reset();
			}
		}

		#region Properties
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries BBMACD
		{
			get { return Values[0]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries BBMACDFrame
		{
			get { return Values[1]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Average
		{
			get { return Values[2]; }
		}

        /// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Upper
		{
			get { return Values[3]; }
		}

        /// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Lower
		{
			get { return Values[4]; }
		}

        /// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries ZeroLine
		{
			get { return Values[5]; }
		}

        /// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries MACDCross
		{
			get { return Values[6]; }
		}

        /// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries BBMACDLine
		{
			get { return Values[7]; }
		}

		/// <summary>
		/// </summary>
		[Description("Band Period for Bollinger Band")]
		[GridCategory("Parameters")]
		public int BandPeriod
		{
			get { return bandperiod; }
			set { bandperiod = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Period for fast EMA")]
		[GridCategory("Parameters")]
		public int Fast
		{
			get { return fast; }
			set { fast = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Period for slow EMA")]
		[GridCategory("Parameters")]
		public int Slow
		{
			get { return slow; }
			set { slow = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Period for smoothing")]
		[GridCategory("Parameters")]
		public int Smooth
		{
			get { return smooth; }
			set { smooth = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Dotsize")]
		[GridCategory("Parameters")]
		public int DotSize
		{
			get { return dotsize; }
			set { dotsize = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of standard deviations")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("# of Std. Dev.")]
		public double StdDevNumber
		{
			get { return stdDevNumber; }
			set { stdDevNumber = Math.Max(0, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Colors")]
		[Gui.Design.DisplayName("MACD Dots Up")]
		public Color DotsUp
		{
			get { return dotsUp; }
			set { dotsUp = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DotsUpSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(dotsUp); }
			set { dotsUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Colors")]
		[Gui.Design.DisplayName("MACD Dots Down")]
		public Color DotsDown
		{
			get { return dotsDown; }
			set { dotsDown = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DotsDownSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(dotsDown); }
			set { dotsDown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Colors")]
		[Gui.Design.DisplayName("MACD Dots Rim")]
		public Color DotsRim
		{
			get { return dotsRim; }
			set { dotsRim = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DotsRimSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(dotsRim); }
			set { dotsRim = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Bollinger Average")]
		public Color BBAverage
		{
			get { return bbAverage; }
			set { bbAverage = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string BBAverageSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(bbAverage); }
			set { bbAverage = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Bollinger Upper Band")]
		public Color BBUpper
		{
			get { return bbUpper; }
			set { bbUpper = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string BBUpperSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(bbUpper); }
			set { bbUpper = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Bollinger Lower Band")]
		public Color BBLower
		{
			get { return bbLower; }
			set { bbLower = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string BBLowerSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(bbLower); }
			set { bbLower = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Zeroline Positive")]
		public Color ZeroPositive
		{
			get { return zeroPositive; }
			set { zeroPositive = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string ZeroPositiveSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(zeroPositive); }
			set { zeroPositive = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Zeroline Negative")]
		public Color ZeroNegative
		{
			get { return zeroNegative; }
			set { zeroNegative = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string ZeroNegativeSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(zeroNegative); }
			set { zeroNegative = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Zero Cross")]
		public Color ZeroCross
		{
			get { return zeroCross; }
			set { zeroCross = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string ZeroCrossSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(zeroCross); }
			set { zeroCross = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Select Color")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Connector")]
		public Color Connector
		{
			get { return connector; }
			set { connector = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string ConnectorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(connector); }
			set { connector = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private anaMACDBBLines[] cacheanaMACDBBLines = null;

        private static anaMACDBBLines checkanaMACDBBLines = new anaMACDBBLines();

        /// <summary>
        /// The anaMACDBBLines (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        public anaMACDBBLines anaMACDBBLines(int bandPeriod, int dotSize, int fast, int slow, int smooth, double stdDevNumber)
        {
            return anaMACDBBLines(Input, bandPeriod, dotSize, fast, slow, smooth, stdDevNumber);
        }

        /// <summary>
        /// The anaMACDBBLines (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        public anaMACDBBLines anaMACDBBLines(Data.IDataSeries input, int bandPeriod, int dotSize, int fast, int slow, int smooth, double stdDevNumber)
        {
            if (cacheanaMACDBBLines != null)
                for (int idx = 0; idx < cacheanaMACDBBLines.Length; idx++)
                    if (cacheanaMACDBBLines[idx].BandPeriod == bandPeriod && cacheanaMACDBBLines[idx].DotSize == dotSize && cacheanaMACDBBLines[idx].Fast == fast && cacheanaMACDBBLines[idx].Slow == slow && cacheanaMACDBBLines[idx].Smooth == smooth && Math.Abs(cacheanaMACDBBLines[idx].StdDevNumber - stdDevNumber) <= double.Epsilon && cacheanaMACDBBLines[idx].EqualsInput(input))
                        return cacheanaMACDBBLines[idx];

            lock (checkanaMACDBBLines)
            {
                checkanaMACDBBLines.BandPeriod = bandPeriod;
                bandPeriod = checkanaMACDBBLines.BandPeriod;
                checkanaMACDBBLines.DotSize = dotSize;
                dotSize = checkanaMACDBBLines.DotSize;
                checkanaMACDBBLines.Fast = fast;
                fast = checkanaMACDBBLines.Fast;
                checkanaMACDBBLines.Slow = slow;
                slow = checkanaMACDBBLines.Slow;
                checkanaMACDBBLines.Smooth = smooth;
                smooth = checkanaMACDBBLines.Smooth;
                checkanaMACDBBLines.StdDevNumber = stdDevNumber;
                stdDevNumber = checkanaMACDBBLines.StdDevNumber;

                if (cacheanaMACDBBLines != null)
                    for (int idx = 0; idx < cacheanaMACDBBLines.Length; idx++)
                        if (cacheanaMACDBBLines[idx].BandPeriod == bandPeriod && cacheanaMACDBBLines[idx].DotSize == dotSize && cacheanaMACDBBLines[idx].Fast == fast && cacheanaMACDBBLines[idx].Slow == slow && cacheanaMACDBBLines[idx].Smooth == smooth && Math.Abs(cacheanaMACDBBLines[idx].StdDevNumber - stdDevNumber) <= double.Epsilon && cacheanaMACDBBLines[idx].EqualsInput(input))
                            return cacheanaMACDBBLines[idx];

                anaMACDBBLines indicator = new anaMACDBBLines();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.BandPeriod = bandPeriod;
                indicator.DotSize = dotSize;
                indicator.Fast = fast;
                indicator.Slow = slow;
                indicator.Smooth = smooth;
                indicator.StdDevNumber = stdDevNumber;
                Indicators.Add(indicator);
                indicator.SetUp();

                anaMACDBBLines[] tmp = new anaMACDBBLines[cacheanaMACDBBLines == null ? 1 : cacheanaMACDBBLines.Length + 1];
                if (cacheanaMACDBBLines != null)
                    cacheanaMACDBBLines.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheanaMACDBBLines = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The anaMACDBBLines (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaMACDBBLines anaMACDBBLines(int bandPeriod, int dotSize, int fast, int slow, int smooth, double stdDevNumber)
        {
            return _indicator.anaMACDBBLines(Input, bandPeriod, dotSize, fast, slow, smooth, stdDevNumber);
        }

        /// <summary>
        /// The anaMACDBBLines (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaMACDBBLines anaMACDBBLines(Data.IDataSeries input, int bandPeriod, int dotSize, int fast, int slow, int smooth, double stdDevNumber)
        {
            return _indicator.anaMACDBBLines(input, bandPeriod, dotSize, fast, slow, smooth, stdDevNumber);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The anaMACDBBLines (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaMACDBBLines anaMACDBBLines(int bandPeriod, int dotSize, int fast, int slow, int smooth, double stdDevNumber)
        {
            return _indicator.anaMACDBBLines(Input, bandPeriod, dotSize, fast, slow, smooth, stdDevNumber);
        }

        /// <summary>
        /// The anaMACDBBLines (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaMACDBBLines anaMACDBBLines(Data.IDataSeries input, int bandPeriod, int dotSize, int fast, int slow, int smooth, double stdDevNumber)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.anaMACDBBLines(input, bandPeriod, dotSize, fast, slow, smooth, stdDevNumber);
        }
    }
}
#endregion
